/*
 * XLife Copyright 1989 Jon Bennett jb7m+@andrew.cmu.edu, jcrb@cs.cmu.edu
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holders make no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include "defs.h"


DoResize()
{
    int owidth=width,oheight=height;
    width = event.xconfigure.width;
    height = event.xconfigure.height;
    xpos+= (owidth - width)/2;
    ypos+= (oheight - height)/2;	
    inputlength = width/FONTWIDTH;
    XResizeWindow(disp,inputw, 
                  width-dispcoord*(COORDW+BORDERWIDTH)-BORDERWIDTH*2,
                  INPUTH);
    XResizeWindow(disp,lifew,width-BORDERWIDTH*2,height-INPUTH-BORDERWIDTH*3);
    XMoveWindow(disp,inputw,0,height-INPUTH-BORDERWIDTH*2);
    XMoveWindow(disp,coordw,width-COORDW-BORDERWIDTH*2,
                            height-INPUTH-BORDERWIDTH*2);
    redrawscreen();
    
}
